% Get figure/ground dataset information.
%
%    fg_data = fg_dataset('train')
%    fg_data = fg_dataset('test')
%
% returns:
%
%    fg_data(n). - structure containing information for image n
%      iid       - image id
%      uids      - list of user ids for each segmentation
%      seg_files - list of human segmentation files
%      fg_files  - for each segmentation, list of f/g annotation files
%                  (fg_files{i} corresponds to seg_files(i))
%      fg_count  - number of f/g annotations for each segmentation
%                  (fg_count(i) = numel(fg_files{i}))
function fg_data = fg_dataset(type, pres)
   % default type - training data
   if (nargin < 1), type = 'train'; end
   % default presentation - color
   if (nargin < 2), pres = 'color'; end
   % set f/g data directory
   fg_data_dir = fullfile('../BSDS300/fg/fgdata',pres);
   % get image ids
   iids = imgList(type);
   % intialize dataset information structure
   n_images = numel(iids);
   e = cell([n_images 1]);
   fg_data = struct('iid',e,'uids',e,'seg_files',e,'fg_files',e,'fg_count',e);
   % lookup human annotations
   for n = 1:n_images
      % get and store current image id
      iid = iids(n);
      fg_data(n).iid = iid;
      % scan for segmentation files
      files = dir(fullfile(bsdsRoot,'human',pres,'*'));
      segs = {};
      uids = {};
      s = 0;
      for i = 1:length(files),
         if ~files(i).isdir, continue; end
         if strcmp(files(i).name,'.'), continue; end
         if strcmp(files(i).name,'..'), continue; end
         uid = sscanf(files(i).name,'%d',1);
         file = segFilename(pres,uid,iid);
         if length(dir(file))==0, continue; end
         s = s + 1;
         segs{s} = file;
         uids{s} = uid;
      end
      % store user ids and segmentation files
      fg_data(n).uids      = uids;
      fg_data(n).seg_files = segs;
      % scan for f/g annotation files
      fg_files = cell([numel(uids) 1]);
      fg_count = zeros([numel(uids) 1]);
      for u = 1:numel(uids)
         fsearch = [fg_data_dir '/' num2str(uids{u}) '/' num2str(iid) '-*.fg'];
         files = dir(fsearch);
         files = {files.name};
         for k = 1:numel(files)
            files{k} = [fg_data_dir '/' num2str(uids{u}) '/' files{k}];
         end
         fg_files{u} = files;
         fg_count(u) = numel(files);
      end
      % store f/g info
      fg_data(n).fg_files = fg_files;
      fg_data(n).fg_count = fg_count;
   end
end
