% compute segmentation and figure/ground output from pb and eigenvectors
function fg_run_output(fg_data, type)
   % default type - training data
   if (nargin < 2), type = 'train'; end
   % set directories
   pb_dir     = ['../data/' type '/pb_components'];
   eigs_dir   = ['../data/' type '/eigs'];
   output_dir = ['../data/' type '/output'];
   % get number of images
   n_images = numel(fg_data);
   % process images
   for n = 1:n_images
      % read image
      iid = fg_data(n).iid;
      im = imgRead(iid,'color');
      sx = size(im,1);
      sy = size(im,2);
      % load pb components
      pb = load([pb_dir '/' num2str(iid) '.mat']);
      % load eigenvectors
      load([eigs_dir '/' num2str(iid) '_eigs.mat']);
      nvec = 16;
      % compute weights
      weights_ae = zeros(size(evals_ae));
      for v = 1:nvec
         if (abs(evals_ae(v)) > 0)
            weights_ae(v) = 1./sqrt(abs(evals_ae(v)));
         end
      end
      % sort by weight
      [junk inds] = sort(weights_ae);
      weights_ae = weights_ae(inds(end:-1:1));
      evecs_ae   = evecs_ae(:,inds(end:-1:1));
      % compute edges
      %evecs = reshape(evecs_ae,[sx sy 1 nvec]);
      %evecs = normalize_evecs(evecs);
      %[er ero] = evecs_to_edges(real(evecs),weights_ae);
      %[ei eio] = evecs_to_edges(imag(evecs),weights_ae);
      %e  = sqrt(er.*er + ei.*ei);
      %eo = sqrt(ero.*ero + eio.*eio);
      % compute edges (exclude first evec)
      evecs2 = reshape(evecs_ae(:,2:end),[sx sy 1 (nvec-1)]);
      evecs2 = normalize_evecs(evecs2);
      [er2 ero2] = evecs_to_edges(real(evecs2),weights_ae(2:end));
      [ei2 eio2] = evecs_to_edges(imag(evecs2),weights_ae(2:end));
      e2  = sqrt(er2.*er2 + ei2.*ei2);
      eo2 = sqrt(ero2.*ero2 + eio2.*eio2);
      % compute gpb for these edges
      %ae_pb_orient  = gPb_from_cues( ...
      %   pb.bg1, pb.bg2, pb.bg3, ...
      %   pb.cga1, pb.cga2, pb.cga3, ...
      %   pb.cgb1, pb.cgb2, pb.cgb3, ...
      %   pb.tg1, pb.tg2, pb.tg3, 0.00056.*eo); % weight learned from calibrate_spb
      ae_pb_orient2 = gPb_from_cues( ...
         pb.bg1, pb.bg2, pb.bg3, ...
         pb.cga1, pb.cga2, pb.cga3, ...
         pb.cgb1, pb.cgb2, pb.cgb3, ...
         pb.tg1, pb.tg2, pb.tg3, 0.0010.*eo2); % weight learned from calibrate_spb
      % enforce nonnegativity
      %ae_pb_orient  = max(0,ae_pb_orient);
      ae_pb_orient2 = max(0,ae_pb_orient2);
      % compute large ucms
      %ucm_ae_lg  = double(contours2ucm(ae_pb_orient.*255,'doubleSize'))./255;
      ucm_ae2_lg = double(contours2ucm(ae_pb_orient2.*255,'doubleSize'))./255;
      % compute ucms
      %ucm_ae =  ucm_ae_lg(3:2:end, 3:2:end);
      ucm_ae2 = ucm_ae2_lg(3:2:end, 3:2:end);
      % save ucms
      %save([output_dir '/ucms/' num2str(iid) '.mat'],'ucm_ae_lg','ucm_ae2_lg','ucm_ae','ucm_ae2');
      save([output_dir '/ucms/' num2str(iid) '.mat'],'ucm_ae2_lg','ucm_ae2');
      % compute per-pixel figure/ground
      fg_ae = reshape(angle(evecs_ae(:,1)),[sx sy]);
      % remap figure/ground to 3 std dev from mean
      fg_m   = mean(fg_ae(:));
      fg_std = std(fg_ae(:));
      ulimit = fg_m + 3.*fg_std;
      llimit = fg_m - 3.*fg_std;
      fg_ae(find(fg_ae > ulimit)) = ulimit;
      fg_ae(find(fg_ae < llimit)) = llimit;
      fg_ae = (fg_ae - min(fg_ae(:)))./(max(fg_ae(:))-min(fg_ae(:)) + eps);
      % create output
      imwrite(im,     [output_dir '/im/' num2str(iid) '.png']);
      %imwrite(ucm_ae, [output_dir '/ucm_norm_ae_1/' num2str(iid) '.bmp']);
      imwrite(ucm_ae2,[output_dir '/ucm_norm_ae2_1/' num2str(iid) '.bmp']);
      %imwrite(1-ucm_ae, [output_dir '/ucm_norm_ae_1_inv/' num2str(iid) '.png']);
      imwrite(1-ucm_ae2,[output_dir '/ucm_norm_ae2_1_inv/' num2str(iid) '.png']);
      imwrite(ind2rgb(round(fg_ae.*255)+1,jet(256)),    [output_dir '/fg_ae_vis/' num2str(iid) '.png']);
   end
end

function evecs = normalize_evecs(evecs)
   [sx sy sz nvec] = size(evecs);
   for n = 1:nvec
      a = abs(evecs(:,:,:,n));
      evecs(:,:,:,n) = evecs(:,:,:,n)./max(a(:));
   end
end
