% create segmentation and figure/ground output at specified threshold
function fg_run_output_th(fg_data, type, th)
   % set directories
   eigs_dir   = ['../data/' type '/eigs'];
   output_dir = ['../data/' type '/output'];
   % get number of images
   n_images = numel(fg_data);
   % process images
   for n = 1:n_images
      % read image
      iid = fg_data(n).iid;
      im = imgRead(iid,'color');
      sx = size(im,1);
      sy = size(im,2);
      % load eigenvectors
      load([eigs_dir '/' num2str(iid) '_eigs.mat']);
      nvec = 16;
      % compute weights
      weights_ae = zeros(size(evals_ae));
      for v = 1:nvec
         if (abs(evals_ae(v)) > 0)
            weights_ae(v) = 1./sqrt(abs(evals_ae(v)));
         end
      end
      % sort by weight
      [junk inds] = sort(weights_ae);
      weights_ae = weights_ae(inds(end:-1:1));
      evecs_ae   = evecs_ae(:,inds(end:-1:1));
      % load ucms
      load([output_dir '/ucms/' num2str(iid) '.mat']);
      % compute per-pixel figure/ground
      fg_ae = reshape(angle(evecs_ae(:,1)),[sx sy]);
      % remap figure/ground to 3 std dev from mean
      fg_m   = mean(fg_ae(:));
      fg_std = std(fg_ae(:));
      ulimit = fg_m + 3.*fg_std;
      llimit = fg_m - 3.*fg_std;
      fg_ae(find(fg_ae > ulimit)) = ulimit;
      fg_ae(find(fg_ae < llimit)) = llimit;
      fg_ae = (fg_ae - min(fg_ae(:)))./(max(fg_ae(:))-min(fg_ae(:)) + eps);
      % segment at fixed threshold
      S = segment_ucm(ucm_ae2_lg.*(ucm_ae2_lg>th));
      % average fg over regions at a fixed threshold
      fg_ae_avg = zeros(size(fg_ae));
      for s = 1:(S.n_regions)
         fg_ae_avg(S.seg_members{s}) = mean(fg_ae(S.seg_members{s}));
      end
      fg_ae_avg = (fg_ae_avg - min(fg_ae_avg(:)))./(max(fg_ae_avg(:))-min(fg_ae_avg(:)) + eps);
      % average image over regions at fixed threshold
      im_r = im(:,:,1);
      im_g = im(:,:,2);
      im_b = im(:,:,3);
      im_r_avg = zeros(size(im_r));
      im_g_avg = zeros(size(im_g));
      im_b_avg = zeros(size(im_b));
      for s = 1:(S.n_regions)
         im_r_avg(S.seg_members{s}) = mean(im_r(S.seg_members{s}));
         im_g_avg(S.seg_members{s}) = mean(im_g(S.seg_members{s}));
         im_b_avg(S.seg_members{s}) = mean(im_b(S.seg_members{s}));
      end
      im_avg = cat(3,im_r_avg,im_g_avg,im_b_avg);
      % create output
      imwrite(ind2rgb(round(fg_ae_avg.*255)+1,jet(256)),[output_dir '/fg_ae_vis_seg/' num2str(iid) '.png']);
      imwrite(im_avg,[output_dir '/seg/' num2str(iid) '.png']);
   end
end
