#include <string.h>
#include <math.h>
#include <mex.h>
#include <assert.h>
#include <stdlib.h>

typedef struct {
  int ind;
  double val;
} t_sort_el;

int comp_sort_el ( const void *a, const void *b ){
  //  t_sort_el *oa = (t_sort_el*)a;
  //  t_sort_el *ob = (t_sort_el*)b;
  if ( ((t_sort_el*)a)->val <  ((t_sort_el*)b)->val ){
    return(0);
  } 
  return(1);
}
int mini(int a, int b){
  if (a<b){
    return(a);
  }
  return(b);
}

void mexFunction (int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[]) {

  // check input arguments, use them to call the real routine

  if ( ( nlhs !=5 ) || ( nrhs != 15 ) ) {
    mexErrMsgTxt("usage [H,c,ind_i,ind_j,spA] = setup_costs_mex(p0,p1,d0,d1,ssd,dist_thresh,use_dist_rank,desc_thresh,use_desc_rank,aweight,areg,dweight,dreg,id1,id2)\n'");
  }

  // set pointers to the arguments

  double* p0 = mxGetPr(prhs[0]);
  double* p1 = mxGetPr(prhs[1]);
  double* d0 = mxGetPr(prhs[2]);
  double* d1 = mxGetPr(prhs[3]);
  int ssd = (int)mxGetScalar(prhs[4]);
  double dist_thresh = mxGetScalar(prhs[5]);
  double use_dist_rank = mxGetScalar(prhs[6]);
  double desc_thresh = mxGetScalar(prhs[7]);
  double use_desc_rank = mxGetScalar(prhs[8]);
  double aweight = mxGetScalar(prhs[9]);
  double areg = mxGetScalar(prhs[10]);
  double dweight = mxGetScalar(prhs[11]);
  double dreg = mxGetScalar(prhs[12]);
  double* id0 = mxGetPr(prhs[13]);
  double* id1 = mxGetPr(prhs[14]);

  int n0 = mxGetM(prhs[0]);
  int n1 = mxGetM(prhs[1]);


  int sn0 = mxGetM(prhs[13]);
  int sn1 = mxGetM(prhs[14]);

  int d = mxGetN(prhs[0]);

  int dd = mxGetN(prhs[2]);

  fprintf(stderr,"n0 = %d  sn0 = %d n1 = %d  sn1 = %d d = %d  dd = %d ",n0,sn0,n1,sn1,d,dd);

  fprintf(stderr,"ssd = %d  dist_thresh = %f dist_rank = %d  ",ssd,dist_thresh,(int)use_dist_rank);
  fprintf(stderr,"desc_thresh = %f desc_rank = %d\n",desc_thresh,(int)use_desc_rank);

  // check argument sizes
  
  if ((n0!=sn0)||(n1!=sn1)){
    mexErrMsgTxt("id size mismatch dimension.\n"); 
  }

  if ( mxGetN(prhs[0]) != mxGetN(prhs[1]) ) { 
    mexErrMsgTxt("p0 and p1 should have the same dimension.\n"); 
  }
  if ( mxGetN(prhs[2]) != mxGetN(prhs[3]) ) { 
    mexErrMsgTxt("d0 and d1 should have the same dimemsion .\n"); 
  }

  
  int *good = new int[n1];
  assert(good);
  fprintf(stderr,"good\n");
  double *row_dists = new double[n1];
  fprintf(stderr,"row_dists\n");
  assert(row_dists);
  t_sort_el *sort_list = new t_sort_el[n1];
  fprintf(stderr,"sort_list\n");
  assert(sort_list);

  double *desc_dists = new double[n1];
  fprintf(stderr,"desc_dists\n");
  assert(desc_dists);
  int *good_inds = new int[n1];
  fprintf(stderr,"good_inds\n");
  assert(good_inds);
  int good_count=0;
  
  int *max_ind_i = new int[n0*n1+n0];
  fprintf(stderr,"max_ind_i\n");
  assert(max_ind_i);
  int *max_ind_j = new int[n0*n1+n0];
  fprintf(stderr,"max_ind_j\n");
  assert(max_ind_j);
  int count=0;

  for (int i=0; i<n0; i++){
    for (int j=0; j<n1; j++){
      row_dists[j]=0;
      good[j]=0;
    }
    for (int k=0; k<d; k++ ){
      double *tp0 = &(p0[n0*k]);
      double *tp1 = &(p1[n1*k]);
      for (int j=0; j<n1; j++){
      	double diff = tp0[i]-tp1[j];
	row_dists[j] = row_dists[j] + diff*diff;
      }
    }
    for (int j=0; j<n1; j++){
      row_dists[j]=sqrt(row_dists[j]);
    }
    if (use_dist_rank){
      for (int j=0; j<n1; j++){
	sort_list[j].ind = j;
	sort_list[j].val = row_dists[j];
      }
      qsort(sort_list, n1, sizeof(t_sort_el), comp_sort_el);
      //      for (int k=0; k<5; k++){
      //	fprintf(stderr,"%d %4.2f ",sort_list[k].ind,sort_list[k].val);
      //      }
      //      fprintf(stderr,"\n");
      int limit = mini((int)dist_thresh,n1);
      for (int j=0; j<limit; j++){
	good[sort_list[j].ind]=1;
      }
    }else{
      for (int j=0; j<n1; j++){
	if (row_dists[j]<dist_thresh){
	  good[j]=1;
	}
      }
    }
    
    good_count = 0;
    for (int j=0; j<n1; j++){
      if (good[j]>0){
	good_inds[good_count] = j;
	good_count = good_count + 1;
      }
    }

    for (int jj=0; jj<good_count; jj++){
      int j = good_inds[jj];
      double diff = 0;
      if (ssd){
	for (int k=0; k<dd; k++){
	  diff = diff + (d0[n0*k+i]-d1[n1*k+j])*(d0[n0*k+i]-d1[n1*k+j]);
	}
	diff=sqrt(diff);
      }else{ // correlation
	for (int k=0; k<dd; k++){
	  diff = diff - (d0[n0*k+i]*d1[n1*k+j]);
	}
	if (id0[i]==1){
	  diff=((diff+1)/80.0)-1.0;
	}
	if (id0[i]!=id1[j]){
	  diff = 5;
	}
      }
      desc_dists[jj]=diff;
    }
    for (int j=0; j<n1; j++){
      good[j]=0;
    }


    if (use_desc_rank){
      for (int j=0; j<good_count; j++){
	sort_list[j].ind = j;
	sort_list[j].val = desc_dists[j];
      }
      qsort(sort_list, good_count, sizeof(t_sort_el), comp_sort_el);
      //      for (int k=0; k<5; k++){
      //	fprintf(stderr,"%d %4.2f ",sort_list[k].ind,sort_list[k].val);
      //      }
      //      fprintf(stderr,"\n");
      int limit = mini((int)desc_thresh, good_count);
      for (int j=0; j<limit; j++){
	good[sort_list[j].ind]=1;
      }
    }else{
      for (int j=0; j<good_count; j++){
	if (desc_dists[j]<desc_thresh){
	  good[j]=1;
	}
      }
    }
    for (int j=0; j<good_count; j++){
      if (good[j]){
	max_ind_i[count]=i;
	max_ind_j[count]=good_inds[j];
	count = count + 1;
      }
    }
    max_ind_i[count]=i;
    max_ind_j[count]=-1;
    count = count + 1;
  }

  int ns = count;

  fprintf(stderr,"count was %d\n",count);

  plhs[0] = mxCreateDoubleMatrix(ns,ns,mxREAL);
  assert(plhs[0]);
  plhs[1] = mxCreateDoubleMatrix(ns,1,mxREAL);
  assert(plhs[1]);
  plhs[2] = mxCreateDoubleMatrix(ns,1,mxREAL);
  assert(plhs[2]);
  plhs[3] = mxCreateDoubleMatrix(ns,1,mxREAL);
  assert(plhs[3]);
  plhs[4] = mxCreateDoubleMatrix(n0,2,mxREAL);
  assert(plhs[4]);
  double* H = mxGetPr(plhs[0]);
  double* c = mxGetPr(plhs[1]);
  double* ind_i = mxGetPr(plhs[2]);
  double* ind_j = mxGetPr(plhs[3]);
  double* spA = mxGetPr(plhs[4]);

  spA[0] = 0;
  int spa_count =0;
  int spa_last=max_ind_i[0];
  for (int i=0; i<count; i++){
    ind_i[i]=max_ind_i[i];
    ind_j[i]=max_ind_j[i];
    if (spa_last!=ind_i[i]){
      spa_last = (int)(ind_i[i]);
      spA[spa_count+n0]=i-1;
      spa_count++;
      spA[spa_count]=i;
      //      fprintf(stderr,"spa_count = %d\n",spa_count);
    }
  }
  spA[spa_count+n0]=count-1;

  double *g_a = new double[d];
  double *g_b = new double[d];
  double *g_c = new double[d];
  double *g_d = new double[d];

  double *v1 = new double[d];
  double *v2 = new double[d];

  double *d_a = new double[dd];
  double *d_b = new double[dd];
  double *d_c = new double[dd];
  double *d_d = new double[dd];


  //      fprintf(stderr,"here0\n");
  for (int i=0; i<(count-1); i++){
    if ((int)(ind_j[i])<0){
      for (int j=0; j<count; j++){
	H[i*count+j]=0;
	H[j*count+i]=0;	
      }
    }else{
      for (int j=0; j<dd; j++){
	d_a[j] = d0[(int)(ind_i[i])+j*n0];
	d_b[j] = d1[(int)(ind_j[i])+j*n1];
      }
      for (int j=0; j<d; j++){
	g_a[j] = p0[(int)(ind_i[i])+j*n0];
	g_b[j] = p1[(int)(ind_j[i])+j*n1];
      }
      double cs = 0;
      if (ssd){
	for (int j=0; j<dd; j++){ 
	  cs = cs + (d_a[j]-d_b[j])*(d_a[j]-d_b[j]);
	}
	cs = sqrt(cs);
      }else{ // correlation
	for (int j=0; j<dd; j++){ 
	  cs = cs - (d_a[j]*d_b[j]);
	}
	if (id0[(int)(ind_i[i])]==1){
	  cs=((cs+1)/80.0)-1.0;
	}
	if (id0[(int)(ind_i[i])]!=id1[(int)(ind_j[i])]){
	  cs = 5;
	}
      }
      c[i]=cs;
      //      fprintf(stderr,"here row %d\n",i);
      for (int j=i+1; j<count; j++){
	//	fprintf(stderr,"here col %d\n",j);
	if (ind_j[j]<0){
	  H[i*count+j]=0;
	  H[j*count+i]=0;
	}else{
	  /*
	  for (int k=0; k<dd; k++){
	    d_c[k] = d0[(int)(ind_i[j])+k*n0];
	    d_d[k] = d1[(int)(ind_j[j])+k*n1];
	  }
	  */
	  for (int k=0; k<d; k++){
	    g_c[k] = p0[(int)(ind_i[j])+k*n0];
	    g_d[k] = p1[(int)(ind_j[j])+k*n1];
	  }
	  double r1 = 0;
	  double r2 = 0;
	  double vd = 0;
	  for (int k=0; k<d; k++){
	    v1[k] = g_c[k]-g_a[k];
	    r1 = r1 + v1[k]*v1[k];
	    v2[k] = g_d[k]-g_b[k];
	    r2 = r2 + v2[k]*v2[k];
	    vd = vd + (v1[k]-v2[k])*(v1[k]-v2[k]);
	  }
	  double aa1 = atan2(v1[1],v1[0]);
	  double aa2 = atan2(v2[1],v2[0]);
          double cost1= 2*3.14159268-fabs(aa1-aa2);
          double cost2= fabs(aa1-aa2);
	  if (cost1>cost2){
	    cost1=cost2;
	  }
	  r1 = sqrt(r1);
	  r2 = sqrt(r2);
	  vd = sqrt(vd);
	  //	  H[i*count+j]=vd/(r1+reg);
	  //	  H[j*count+i]=vd/(r1+reg);
          if (r1<0.0001){
	    double adist = aweight*10000;    
	    H[i*count+j]=adist;    
	    H[j*count+i]=adist;
	  }else{
	    double adist = aweight*cost1/(r1+areg);    
	    H[i*count+j]=adist;
	    H[j*count+i]=adist;
	  }


          if (r1<0.0001){
            double ddist = dweight*10000;
	    H[i*count+j]+=ddist;    
	    H[j*count+i]+=ddist;
	  }else{
	    double ddist = dweight*fabs(r1-r2)/(r1+dreg);
	    H[i*count+j]+=ddist;
	    H[j*count+i]+=ddist;
	  }


	  /*
	  if ((vd/(r1+reg))>3){
	    double extra_stuff = (vd/(r1+reg))-3;
	    extra_stuff = sqrt(extra_stuff*extra_stuff);
//	    extra_stuff = 10000;
	    H[i*count+j] +=   extra_stuff;
	    H[j*count+i] +=   extra_stuff;
	  }
	  */


	  /*
	  if ((vd/(r1+reg))>2){
	    double extra_stuff = ((vd/(r1+reg))-2);
	    extra_stuff = extra_stuff*extra_stuff;
	    H[i*count+j] +=   extra_stuff;
	    H[j*count+i] +=   extra_stuff;
	  }
	  */
	}
      }
    }
  }

  delete v1;
  delete v2;
  delete g_a;
  delete g_b;
  delete g_c;
  delete g_d;
  delete d_a;
  delete d_b;
  delete d_c;
  delete d_d;
  delete desc_dists;
  delete good_inds;
  delete max_ind_i;
  delete max_ind_j;
  delete good;
  delete row_dists;
  delete sort_list;
}
