% Normalized cuts.
% Return a set of ncut eigenvectors along with corresponding weights.
function [evecs weights] = ncut(W, nvec)
   % specify default number of eigenvectors
   if (nargin < 2), nvec = min(length(W)-1,8); end
   % specify options
   opts.issym=1;
   opts.isreal = 1;
   opts.disp=2;
   % guarantee W is symmetric
   W = max(W,W.');
   % compute degree matrix
   D = diag(sum(W,1));
   % compute eigenvectors
   [evecs evals] = eigs(D - W, D, nvec+1, 'sm', opts);
   evals = diag(evals);
   % reorder eigenvectors by importance
   evecs(:,1:end) = evecs(:,end:-1:1);
   evals(1:end) = evals(end:-1:1);
   % remove first eigenvector
   evecs = evecs(:,2:end);
   evals = evals(2:end);
   % normalize eigenvectors
   for n = 1:nvec
      e_min = min(evecs(:,n));
      e_max = max(evecs(:,n));
      evecs(:,n) = (evecs(:,n) - e_min)./(e_max - e_min + eps);
   end
   % compute weights
   weights = zeros([nvec 1]);
   for n = 1:nvec
      if (evals(n) > 0)
         weights(n) = 1./sqrt(evals(n));
      end
   end
   % normalize weights
   weights = weights./(sum(weights));
end
